--only startup if player is druid
if Symbiosis.stop then
	return
end

local symbdebug = false;--TODO: remove on live realms

--define locals
local SymbiosisButton, pop, targetstring, durationstring, BuffFrame, slider, LastSpec;
local Member, ButtonUnit, Target = {}, {}, {};

local Lasttimer = GetTime();
local Timerintervall = 5;--this is the intervall (in seconds) in which we check the buff on target
local BadTimeStart = 15;--when the duration string should be yellow (in minutes)

local LastWhisper = 0;
local WhisperDelay = 60;--delay between two whispers (in seconds?)
local WhisperTimeOut = 40;--time when unregistering event to check for successfull buff cast
local LastWhisperTarget = "-";

local membercount, wantedmembercount = 0,0;
local UpdateAfterCombat, RemoveAfterCombat = false, false;
local groupsize = "solo";

local SymbiosisIcon = Symbiosis.SymbiosisIcon;
local SymbiosisSpell = Symbiosis.SymbiosisSpell;
local SymbiosisSpellID = Symbiosis.SymbiosisSpellID;

--button layout stuff
local maxmemberbuttons = 10;
local PopUpTagHeight, PopUpTagWidth = 10,90;
local PopUpMaxButtonHeight, PopUpMaxButtonWidth = 50, 150;
local PopUpMaxIconHeight, PopUpMaxIconWidth = PopUpMaxButtonHeight, 50;
local PopUpButtonHeight, PopUpButtonWidth = 50, 150;
local PopUpIconHeight, PopUpIconWidth = PopUpButtonHeight-10, PopUpMaxIconWidth-10;

--local optimizers
local UnitBuff = UnitBuff;
local UnitGUID = UnitGUID;
local UnitName = UnitName;
local UnitIsConnected = UnitIsConnected;
local UnitClass = UnitClass;
local UnitIsDeadOrGhost = UnitIsDeadOrGhost;
local GetNumGroupMembers = GetNumGroupMembers;
local GetRaidRosterInfo = GetRaidRosterInfo;
local GetSpecialization = GetSpecialization;
local GetSpellInfo = GetSpellInfo;
local GetTime = GetTime;
local floor = floor;

--spell lists, defined in SymbiosisSpells.lua
local SpellsGot = Symbiosis.SpellsGot;
local SpellsGranted = Symbiosis.SpellsGranted;

-----------------------------------------
--helper functions
-----------------------------------------

--check if we are in raid or party
local function CheckRaidOrParty()
	if IsInRaid() then
		--in raid
		groupsize = "raid";
		membercount = GetNumGroupMembers();
	elseif IsInGroup() then
		--in party
		groupsize = "party";
		membercount = GetNumGroupMembers();
	else
		--not in group
		groupsize = "solo";
		membercount = 0;
	end;
end;

local function ShowHideButton(DoShow)
	if DoShow then
		SymbiosisButton:Show();
	else
		SymbiosisButton:Hide();
	end;
end;

-----------------------------------------
--timer (for buff watch) stuff
-----------------------------------------
--update duration of symbiosis buff to durationstring
local function UpdateDuration(Time)
	if ( Time > 0 ) then
		if ( (Time/60) < BadTimeStart ) then
			--yellow (badtimestart=15)
			durationstring:SetTextColor(GameFontNormal:GetTextColor());
		else
			--green
			durationstring:SetTextColor(0.4,1,0);
		end;
		if ( (-floor(-Time/60)) > 1 ) then
			--display minutes (green or yellow when badtimestart(s))
			durationstring:SetText("Dur.: "..(-floor(-Time/60)).." Min");--loc
		else
			--display seconds (red)
			durationstring:SetTextColor(1,0,0);
			durationstring:SetText("Dur.:"..floor(Time).." Sec");--loc
		end;
	elseif ( Time == -1 ) then
		--buff is not up (red)
		durationstring:SetTextColor(1,0,0);
		durationstring:SetText("Buff not up!");--loc
	elseif ( Time == -2) then
		--target has buff of other druid
		durationstring:SetText("Target stolen!");--loc
		durationstring:SetTextColor(0,0,1);--blue
	elseif ( Time == 0 ) then
		--taget is out of scan range
		durationstring:SetText("Buff is up");
		durationstring:SetTextColor(GameFontNormal:GetTextColor());
	else
		--this should never happen
		durationstring:SetText("");
	end;
end;

--timer for buff duration
local CheckBuffSetTimerAndSetDuration;
local function SetTimer(StartTimer)
	if StartTimer then
		SymbiosisButton:SetScript("OnUpdate",function()
			if GetTime()-Lasttimer > Timerintervall then
				Lasttimer = GetTime();
				CheckBuffSetTimerAndSetDuration(false);
			end;
		end);
	else
		--remove onupdate handler
		SymbiosisButton:SetScript("OnUpdate",nil);
	end;
end;

--Check current buff on Target["unit"], set duration and start timer if necessary.
--ToggleTimerOn: 
--SetTimer() calls ToggleTimerOn==false
--SetSymbiosisButton() calls ToggleTimerOn==true
--BuffFrame_OnEvent() calls ToggleTimerOn==true
function CheckBuffSetTimerAndSetDuration(ToggleTimerOn)--this is local, decleration is above SetTimer()
	if Target["unit"] then
		local _,_,_,_,_,_,expiration,unitCaster = UnitBuff(Target["unit"],SymbiosisSpell);
		if expiration then
			if unitCaster == "player" then
				--target has our buff
				UpdateDuration(expiration - GetTime());
			elseif unitCaster then
				--target has buff of other druid
				UpdateDuration(-2);
			else
				--target is out of scan range
				UpdateDuration(0);
			end;

			--if we are not called by SetTimer then call SetTimer(true)
			if ToggleTimerOn then
				SetTimer(true);
			end;
		else
			--remove timer if target does not have any symbiosis buff
			SetTimer(false);
			UpdateDuration(-1);
		end;
	else
		--remove timer if target is nil
		SetTimer(false);
		UpdateDuration(-1);		
	end;
end;

-----------------------------------------
--configure symbiosis main button
--and setup table for spec icons
-----------------------------------------

--color to display the name of a member, based on class
local function GetColor(class)
	if RAID_CLASS_COLORS[class] then
		local r,g,b = RAID_CLASS_COLORS[class]["r"], RAID_CLASS_COLORS[class]["g"], RAID_CLASS_COLORS[class]["b"];
		return r,g,b;
	else
		return GameFontNormal:GetTextColor();
	end
end

local function GetIconOfSpec(TargetClass)
	local function CheckSpec(SpecIconTable,theclass)
		local _,_,SpellIcon = GetSpellInfo(SpecIconTable[theclass]);
		if SpellIcon then
			return SpellIcon;
		else
			return false;
		end;
	end;

	return CheckSpec(SpellsGot[GetSpecialization()],TargetClass),SpellsGot[GetSpecialization()][TargetClass];
end;

--set the icon of the symbiosis main button based on spec of druid and class of target
local function SetSymbiosisButtonIcon()
	local icon;
	if Target["class"] then
		icon = GetIconOfSpec(Target["class"]);
	end;

	if icon then
		SymbiosisButton.icon:SetTexture(icon);
	elseif Target["class"] then
		SymbiosisButton.icon:SetTexture("Interface\\GLUES\\CHARACTERCREATE\\UI-CHARACTERCREATE-CLASSES");
		local coords = CLASS_ICON_TCOORDS[Target["class"]];
		SymbiosisButton.icon:SetTexCoord(unpack(coords));
	elseif Target["name"] then
		SymbiosisButton.icon:SetTexture("Interface\\ICONS\\INV_Misc_QuestionMark");
	else
		SymbiosisButton.icon:SetTexture(SymbiosisIcon);
	end
end;

--set name above the symbiosis main button in color of class
local function SetSymbiosisButtonName()
	targetstring:SetText(Target["name"]);
	targetstring:SetTextColor(GetColor(Target["class"]));
end;

--add icon of spell/class to symbiosis button based on the Target[] table and set name
local function SetSymbiosisButton()
	SetSymbiosisButtonIcon();
	SetSymbiosisButtonName();
	CheckBuffSetTimerAndSetDuration(true);

	BuffFrame:RegisterEvent("UNIT_AURA");
end;

function Symbiosis.RemoveSymbiosisTarget()
	BuffFrame:UnregisterAllEvents();
	SetTimer(false);

	Target = {};

	SymbiosisButton.icon:SetTexture(SymbiosisIcon);
	SymbiosisButton.icon:SetTexCoord(0,1,0,1);

	durationstring:SetText("");

	targetstring:SetText("No Target");--loc
	targetstring:SetTextColor(GameFontNormal:GetTextColor());

	if UnitAffectingCombat("player") then
		RemoveAfterCombat = true;
	else
		SymbiosisButton:SetAttribute("macrotext","");
	end;
end;

-----------------------------------------
--setup stuff to display popup functions
-----------------------------------------
local ShowPopUp;

--icon to display near the member, based on druid spec and target class
local function SetIcon(frame,i)
	local icon,spellId;
	if Member[i]["class"] then
		icon, spellId = GetIconOfSpec(Member[i]["class"]);
	end;
	if icon and spellId then
		--set icon
		frame:SetTexture(icon);
		
		local IconParent = frame:GetParent();
		--set on enter event
		IconParent:SetScript("OnEnter",function()
			GameTooltip:SetOwner(IconParent,"ANCHOR_CURSOR");
			GameTooltip:SetHyperlink("spell:"..spellId);
			GameTooltip:Show();
		end);
		
		--set on leave event
		IconParent:SetScript("OnLeave",function()
			GameTooltip:Hide();
		end);
	else
		--we have no class of target or couldnt retrieve icon for other reasons
		if Member[i]["class"] then
			frame:SetTexture("Interface\\GLUES\\CHARACTERCREATE\\UI-CHARACTERCREATE-CLASSES");
			local coords = CLASS_ICON_TCOORDS[Member[i]["class"]];
			frame:SetTexCoord(unpack(coords));
		else
			frame:SetTexture("Interface\\ICONS\\INV_Misc_QuestionMark");
		end;
	end
end;

--delete OnMouseDown event of button
local function ClearEvent(i)
	_G["SymbiosisButton_MemberButton_"..i]:SetScript("OnMouseDown",nil);
end;

--clear button for when player is not in raid or party
local function ClearButton(i,text)
	local mybutton = _G["SymbiosisButton_MemberButton_"..i];
	mybutton.fontstring:SetText(text);
	mybutton.fontstring:SetTextColor(GameFontNormal:GetTextColor());
	_G["SymbiosisButton_MemberButtonIcon_"..i].icon:SetTexture("");
	
	local mytag = _G["SymbiosisButton_TagFrame_"..i];
	mytag:SetText("");
	mytag:SetTextColor(GameFontNormal:GetTextColor());
	
	ClearEvent(i);
end;

local DoWhisper = false;
local BuffScanStartTime = 0;
local unitCaster;
local WhisperFrame = CreateFrame("Frame");
WhisperFrame:SetScript("OnEvent",function()
	if Target["unit"] then
		_,_,_,_,_,_,_,unitCaster = UnitBuff(Target["unit"],SymbiosisSpell);
	else
		unitCaster = nil;
	end;
	
	if unitCaster then
		if unitCaster == UnitName("player") then
			DoWhisper = true;
			if LastWhisperTarget == Target["name"] then
				if (GetTime()-LastWhisper) < WhisperDelay then
					DoWhisper = false;
				end
			end;
			
			if DoWhisper then
				LastWhisperTarget = Target["name"];
				LastWhisper = GetTime();
				--SendChatMessage("DEBUG: This is a test "..Symbiosis.GetWhisperMessage(4,Target["class"]),"WHISPER",nil,"Hoochie");--TODO: replace Hoochie with Target["name"] and remove debug when everything is working.
				print("Symbiosis_DEBUG: Whisper sent.");
			end;
			
			WhisperFrame:UnregisterAllEvents();
		end;
	end;
	
	if (GetTime() - BuffScanStartTime) > WhisperTimeOut then
		WhisperFrame:UnregisterAllEvents();
	end;
end);

--the event fired when symbiosis button is clicked
local function OnClickEventMainButton(self,button)
	--whisper stuff
	if (button == "LeftButton") and (SymbConfig["WhisperEnable"]) and Target["name"] then
		WhisperFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
		BuffScanStartTime = GetTime();
	end;
	
	--the following is the standard button mapping
	if not UnitAffectingCombat("player") then
		if button == "RightButton" then
			if pop:IsShown() then
				pop:Hide();
			else
				ShowPopUp();
			end;
		end;
	else
		if button == "RightButton" then
			print("SymbiosisButton: Cannot change Symbiosis target in combat.");--loc and rewrite message
		end;
	end
	
	--notice for: remove symb buff on shift left click (the actual remove is at setattribute of secure button)
	if SymbConfig["DisableBuffRemove"] == false then
		if (button == "LeftButton") and IsShiftKeyDown() then
			if UnitBuff("player",SymbiosisSpell) then
				print("Symbiosis: Removed Symbiosis buff because you shift-left-clicked the Symbiosis button.");--loc
			end;
		end;
	end;

	--on first login print name of addon
	if SymbConfig["firstlogin"] then
		print("Symbiosis: You just clicked the Symbiosis button for the first time. To configure this addon type '/symbiosis config'.");--loc
		SymbConfig["firstlogin"] = false;
	end;
end;

--set the onmousedown event of i-th button
local function SetClickEvent(buttonID)
	if UnitAffectingCombat("player") then
		pop:Hide();
		print("Symbiosis: You are in combat!");--loc
	else
		_G["SymbiosisButton_MemberButton_"..buttonID]:SetScript("OnMouseDown",function()
			Target = ButtonUnit[buttonID];

			SetSymbiosisButton();
			
			--add macro attribute with cast command with current target
			SymbiosisButton:SetAttribute("macrotext","/cast [@"..Target["unit"].."] "..SymbiosisSpell);
			
			--set whisper command
			SymbiosisButton:SetScript("OnMouseDown",OnClickEventMainButton);
			
			pop:Hide();
		end);
	end;
end;

--set the name (in class color) and the icon (if target has our buff show little symbiosis icon) in the popup menu
local function SetNameAndIcon(i,buttonID)
	local name, online, isdead, class, unit = Member[i]["name"], Member[i]["online"], Member[i]["isdead"], Member[i]["class"], Member[i]["unit"];
	local mybutton = _G["SymbiosisButton_MemberButton_"..buttonID];
	local mytag = _G["SymbiosisButton_TagFrame_"..buttonID];
	local tag = "";
	local icon = "";
	SetIcon(_G["SymbiosisButton_MemberButtonIcon_"..buttonID].icon,i);
	if online then
		mybutton.fontstring:SetTextColor(GetColor(class));
		if isdead then
			tag = " (".."dead"..")";--loc
		end
		_G["SymbiosisButton_MemberButtonIcon_"..buttonID].icon:SetDesaturated(nil);
	else
		mybutton.fontstring:SetTextColor(0.5,0.5,0.5);
		tag = " (".."d/c"..")";--loc
		_G["SymbiosisButton_MemberButtonIcon_"..buttonID].icon:SetDesaturated(1);
	end
	local _,_,_,_,_,_,_,unitCaster = UnitBuff(unit,SymbiosisSpell);
	if unitCaster then
		if unitCaster == "player" then
			tag = " (".."buffed"..")";--loc
			icon = "|T"..SymbiosisIcon..":20|t ";
		else
			mybutton.fontstring:SetTextColor(0.67,0.67,0.67);
			tag = " (".."stolen"..")";--loc
			_G["SymbiosisButton_MemberButtonIcon_"..buttonID].icon:SetDesaturated(1);
		end;
	end;
	
	--Set name and if target has our buff set little symbiosis icon:
	mybutton.fontstring:SetText(icon..name);
	
	--set tags:
	mytag:SetText(tag);
	mytag:SetTextColor(mybutton.fontstring:GetTextColor());
	local myTagContainer = mytag:GetParent();
	myTagContainer:ClearAllPoints();
	if icon == "" then
		myTagContainer:SetPoint("BOTTOMLEFT",mybutton,"BOTTOMLEFT",10,7);
		mytag:SetJustifyH("LEFT");
	else
		myTagContainer:SetPoint("TOP",mybutton.fontstring,"BOTTOM",5,2);
		mytag:SetJustifyH("CENTER");
	end;
	
	--fill up ButtonUnit table
	ButtonUnit[buttonID] = {};
	ButtonUnit[buttonID] = Member[i];
end;

--shows number of buttons we need (how many member in party) and hides the rest
local function SetButtonNumber(count)
	if count < maxmemberbuttons then
		_G["SymbiosisButton_MemberButton_"..count+1]:Hide();
	else
		count = maxmemberbuttons;
	end;
	pop:SetHeight(PopUpMaxButtonHeight*count);
	for i=1,count do
		_G["SymbiosisButton_MemberButton_"..i]:Show();
	end;
end;

--setup number of party members
--returns: false (not in party or raid) or true (is in party/raid and members were recorded)
local function SetupMemberList()
	--delete old stuff
	wipe(Member);
	
	--check if we are in raid or party
	CheckRaidOrParty();

	if groupsize == "solo" then
		--we are not in raid and not in party
		return false;
	else
		--setup table
		local UnwantedUnitCount = 0;
		wantedmembercount = 0;
		local AllFiltered = true;
		local AllDruid = true;
		
		
		if groupsize ~= "raid" then
			membercount = membercount - 1;--units in party only go up to "party4"
		end;
		
		for i=1,membercount do
			local name,subgroup,class,online,isDead;
			if groupsize == "raid" then
				name,_,subgroup,_,_,class,_,online,isDead = GetRaidRosterInfo(i);
				if not name then
					if symbdebug then
						print("DEBUG: Name was nil.");
					end;
					name = "Unknown";--loc
					
					if UnitClass(groupsize..i) then
						class = UnitClass(groupsize..i);
					else
						class = "DRUID";
					end;
					
					if not subgroup then
						subgroup = 1;
					end;
					
					online = true;
					isDead = false;
				end;
			else
				local myname,myrealm = UnitName(groupsize..i);
				if myrealm and (myrealm ~= "") then
					name = myname..'-'..myrealm;
				else
					name = myname;
				end;
				subgroup = 1;
				_,class = UnitClass(groupsize..i);
				isDead = UnitIsDeadOrGhost(groupsize..i);
				online = UnitIsConnected(groupsize..i);
			end;
			--Check if unit is dead and we should hide dead
			local AddUnit = (not (SymbConfig["DontShowUnitDead"] and isDead));
			
			--Check if unit is offline and we should hide offline
			AddUnit = AddUnit and (not (SymbConfig["DontShowUnitOffline"] and (not online)));
						
			--Check if unit is stolen
			if SymbConfig["DontShowUnitStolen"] then
				local BuffIsUp,_,_,_,_,_,_,unitCaster = UnitBuff(groupsize..i,SymbiosisSpell);
				if (BuffIsUp) and (unitCaster ~= "player") then
					AddUnit = false;
				end;
			end;

			--Check if unit is in insignificant group
			if SymbConfig["DontShowUnitInsignificant"] then
				--if in 5man party: ignore everything above grp1
				if (groupsize == "party") and (subgroup > 1) then
					AddUnit = false;
				end;
				
				local size = GetInstanceDifficulty();--returns 1(5/10/40 normal), 2(5hc/25normal), 3(10hc) or 4(25hc)
								
				--if in 10man raid: ignore everything above grp2
				if (groupsize == "raid") and ((size == 1) or (size == 3)) and (subgroup>2) then
					AddUnit = false;
				end;
				
				--if in 25man raid: ignore everything above grp5
				if (groupsize == "raid") and ((size == 2) or (size == 4)) and (subgroup>5) then
					AddUnit = false;
				end;
			end;
			
			--If we wont add unit because of filter notice the user (if enabled)
			if AddUnit and (class ~= "DRUID") then
				AllFiltered = false;
			end;
			
			--If all raid members are druid dont notice the user
			if class ~= "DRUID" then
				AllDruid = false;
			end;

			--Check if unit is druid
			AddUnit = AddUnit and (class ~= "DRUID");
			if AddUnit then
				local GUID = UnitGUID(groupsize..i);
				Member[i-UnwantedUnitCount] = {};
				Member[i-UnwantedUnitCount]["class"] = class;
				Member[i-UnwantedUnitCount]["GUID"] = GUID;
				Member[i-UnwantedUnitCount]["unit"] = groupsize..i;
				Member[i-UnwantedUnitCount]["name"] = name;
				Member[i-UnwantedUnitCount]["online"] = online;
				Member[i-UnwantedUnitCount]["isdead"] = isdead;
				wantedmembercount = wantedmembercount + 1;
			else 
				UnwantedUnitCount = UnwantedUnitCount + 1;
			end
		end
		
		if AllDruid then
			AllFiltered = false;
		end;
		return true,AllFiltered;
	end;
end;

--show PopUp menu when main button was right clicked. this is local, declaration is at the beginning of "setup stuff to display popup functions"
function ShowPopUp()
	local xofs, yofs = 8,8;

	pop:Show();

	--look in which quadrant we are and adjust position of the PopUp accordingly
	pop:ClearAllPoints();
	if SymbiosisButton:GetLeft() > UIParent:GetWidth()/2 then
		if SymbiosisButton:GetBottom() > UIParent:GetHeight()/2 then
			pop:SetPoint("TOPRIGHT",SymbiosisButton,"BOTTOMLEFT",-xofs,-yofs);--pop ist links unten
		else
			pop:SetPoint("BOTTOMRIGHT",SymbiosisButton,"TOPLEFT",-xofs,yofs);--pop ist links oben
		end
	else
		if SymbiosisButton:GetBottom() > UIParent:GetHeight()/2 then
			pop:SetPoint("TOPLEFT",SymbiosisButton,"BOTTOMRIGHT",xofs,-yofs);--pop ist rechts unten
		else
			pop:SetPoint("BOTTOMLEFT",SymbiosisButton,"TOPRIGHT",xofs,yofs);--pop ist rechts oben
		end
	end
	local SetupSuccess, AllFiltered = SetupMemberList();
	if SetupSuccess then
		if (wantedmembercount > 0) then
			--adjust height according to member count and fill PopUp with members
			local count,DoSlider;
			if wantedmembercount > maxmemberbuttons then
				count = maxmemberbuttons;
				DoSlider = true;
			else
				count = wantedmembercount;
				DoSlider = false;
			end
			SetButtonNumber(count);

			--setup first set of buttons
			for i=1,count do
				SetNameAndIcon(i,i);
				SetClickEvent(i);
			end

			--setup slider if needed
			if DoSlider then
				slider:SetMinMaxValues(1,wantedmembercount-maxmemberbuttons+1);
				slider:SetValue(1);
				slider:SetValueStep(1);
				slider:Show();

				slider:SetScript("OnValueChanged",function()
					_G[slider:GetName().."Low"]:SetText(slider:GetValue());
					local buttonnumber = 1;
					for i=slider:GetValue(),slider:GetValue()+maxmemberbuttons-1 do
						SetNameAndIcon(i,buttonnumber);
						SetClickEvent(buttonnumber);
						buttonnumber = buttonnumber + 1;
					end;
				end);
			else
				slider:Hide();
			end;
		else
			--there is no vailable target
			slider:Hide();
			SetButtonNumber(1);
			_G["SymbiosisButton_TagFrame_1"]:SetText("");
			ClearButton(1,"No target available");--loc
			if AllFiltered and true then--TODO: add option here (disable notification when allfiltered)
				print("Symbiosis: Your filters removed all targets.");--loc
			end;
		end
	else
		--remove everything from list and tell the user that he is not in a party/raid
		slider:Hide();
		SetButtonNumber(1);
		_G["SymbiosisButton_TagFrame_1"]:SetText("");
		ClearButton(1,"Not in party");--loc
	end
end;

-----------------------------------------
--create functions
-----------------------------------------

--create slider for popup
local function CreateGroupSlider()
	slider = Symbiosis.CreateSlider(0,0,"",1,15,1,0,pop);

	slider:ClearAllPoints();
	slider:SetPoint("TOPLEFT",pop,"BOTTOMLEFT",0,-5);
	_G[slider:GetName() .. "Low"]:SetText("");
	_G[slider:GetName() .. "High"]:SetText("");

	slider:SetScript("OnValueChanged",function()
		_G[slider:GetName().."Low"]:SetText(slider:GetValue());
	end);

	local function MouseWheelFunc(self,direction)
		if slider:IsVisible() then
			--direction: 1 for up, -1 for down
			slider:SetValue(slider:GetValue()-direction);
		end;
	end;
	
	slider:SetScript("OnMouseWheel",function(self,direction)
		MouseWheelFunc(self,direction);
	end);

	for i=1,maxmemberbuttons do
		local mybutton = _G["SymbiosisButton_MemberButton_"..i];
		mybutton:EnableMouseWheel(1);
		mybutton:SetScript("OnMouseWheel",function(self,direction)
			MouseWheelFunc(self,direction);
		end);
	end;
	
	slider:Hide();
end;

--create our buttons in the PopUp menu
local function CreateMemberButtons()	
	local lastbutton = pop;
	for i=1,maxmemberbuttons do
		--create the actual button
		local buttonframe = CreateFrame("Frame","SymbiosisButton_MemberButton_"..i,lastbutton);
		buttonframe:SetWidth(lastbutton:GetWidth());
		buttonframe:SetHeight(PopUpButtonHeight);
		buttonframe:SetPoint("TOPLEFT",pop,"TOPLEFT",0,-(i-1)*PopUpButtonHeight);
		buttonframe:SetFrameStrata("MEDIUM");--this must be "medium" and pop must be "high"
		buttonframe:Show();

		--create the title of each button (which will be the names of the member)
		buttonframe.fontstring = Symbiosis.CreateLabel(buttonframe,i,buttonframe:GetWidth()-50-10);--50 for class icon and 10 for left margin
		buttonframe.fontstring:SetHeight(20);
		buttonframe.fontstring:ClearAllPoints();
		buttonframe.fontstring:SetPoint("LEFT",buttonframe,"LEFT",10,0);
		buttonframe:SetBackdrop({	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
									edgeFile = "Interface/ArenaEnemyFrame/UI-Arena-Border",
									tile = true,
									tileSize = 1,
									edgeSize = 1});

		--create tag container
		local tagcontainer = CreateFrame("Frame","SymbiosisButton_TagContainer_"..i,buttonframe);
		tagcontainer:SetWidth(PopUpTagWidth);
		tagcontainer:SetHeight(PopUpTagHeight);
		tagcontainer:SetPoint("TOP",buttonframe.fontstring,"BOTTOM",5,2);
		
		--create tag, e.g. for dc/offline/stolen
		local tagframe = tagcontainer:CreateFontString("SymbiosisButton_TagFrame_"..i,"ARTWORK","GameFontNormal");
		tagframe:SetAllPoints(tagcontainer);
		
		--create container for the icon
		local iconcontainer = CreateFrame("Frame","SymbiosisButton_MemberButtonIconContainer_"..i,buttonframe);
		iconcontainer:SetWidth(PopUpMaxIconWidth);
		iconcontainer:SetHeight(PopUpMaxIconHeight);
		iconcontainer:SetPoint("TOPRIGHT",buttonframe,"TOPRIGHT");

		--create the icon next to the name		
		local iconframe = CreateFrame("Frame","SymbiosisButton_MemberButtonIcon_"..i,iconcontainer);
		iconframe:SetWidth(PopUpIconWidth);
		iconframe:SetHeight(PopUpIconHeight);
		iconframe:SetPoint("CENTER",iconcontainer,"CENTER");
		iconframe.icon = iconframe:CreateTexture(nil,"CENTER");
		iconframe.icon:SetAllPoints(iconframe);
		
		lastbutton = buttonframe;
	end
end

--create PopUp menu
local function CreatePopUpMenu()
	pop = CreateFrame("Frame","SymbiosisButton_PopUpFrame",SymbiosisButton);
	
	--hide frame on esc pressed
	tinsert(UISpecialFrames,pop:GetName());
	
	pop:SetWidth(PopUpButtonWidth);
	
	--frame layout
	pop:SetBackdrop({	edgeFile = "Interface/ArenaEnemyFrame/UI-Arena-Border",
						tile = true,
						tileSize = 3,
						edgeSize = 3});

	--this must be "high" and buttons must be "medium"
	pop:SetFrameStrata("HIGH");
	
	pop:Hide();
end;

--create text above main button
local function CreateTargetString()
	targetstring = Symbiosis.CreateLabel(SymbiosisButton,"No Target",200,0,35);--loc
	targetstring:SetJustifyH("CENTER");
	targetstring:ClearAllPoints();
	targetstring:SetPoint("BOTTOM",SymbiosisButton,"TOP",0,4);
	targetstring:SetHeight(10);

	durationstring = Symbiosis.CreateLabel(SymbiosisButton,"",100,0,0);
	durationstring:SetJustifyH("CENTER");
	durationstring:ClearAllPoints();
	durationstring:SetPoint("TOP",SymbiosisButton,"BOTTOM",0,16);
end;

--create main button
local function CreateMainButton()	
	--create button, set texture and show the button
	SymbiosisButton = CreateFrame("Button","SymbiosisButton",UIParent,"SecureActionButtonTemplate");
	SymbiosisButton:SetWidth((SymbButton["Size"]/100)*Symbiosis.ButtonSize);
	SymbiosisButton:SetHeight(SymbiosisButton:GetWidth());
	_G[Symbiosis.SizerSlider:GetName() .. "Text"]:SetText("Size: " .. SymbButton["Size"].."%");--loc
	Symbiosis.SizerSlider:SetValue(SymbButton["Size"]);
	SymbiosisButton.icon = SymbiosisButton:CreateTexture(nil,"CENTER");
	SymbiosisButton.icon:SetTexture(SymbiosisIcon);
	SymbiosisButton.icon:SetAllPoints(SymbiosisButton);
	SymbiosisButton:SetPoint(
		SymbButton["GetPoint_pos1"],
		UIParent,
		SymbButton["GetPoint_pos2"],
		SymbButton["GetPoint_x"],
		SymbButton["GetPoint_y"]
	);
	ShowHideButton(true);
	
	--hide tooltip when mousing over button
	SymbiosisButton:SetScript("OnEnter",function()
		GameTooltip:Hide();
	end);
	
	--cast buff on unmodified leftclick
	SymbiosisButton:SetAttribute("type1","macro");
	
	--remove buff on shift leftclick
	if SymbConfig["DisableBuffRemove"] == false then
		SymbiosisButton:SetAttribute("shift-type1","cancelaura");
	end;
	SymbiosisButton:SetAttribute("unit","player");
	SymbiosisButton:SetAttribute("spell",SymbiosisSpell);
	
	--PopUp window on rightclick
	SymbiosisButton:SetScript("OnMouseDown",OnClickEventMainButton);

	--setup stuff to drag button
	SymbiosisButton:EnableMouse(true);
	SymbiosisButton:SetMovable(true);
	SymbiosisButton:RegisterForDrag("LeftButton");
	SymbiosisButton:SetScript("OnDragStop",function()
		SymbiosisButton:StopMovingOrSizing();
		local pos1,_,pos2,x,y = SymbiosisButton:GetPoint(1)
		SymbButton["GetPoint_pos1"] = pos1;
		SymbButton["GetPoint_pos2"] = pos2;
		SymbButton["GetPoint_x"] = x;
		SymbButton["GetPoint_y"] = y;
	end);
	
	--create dragstop frame: green "click me to disable dragging of button" button above symbiosis button
	SymbiosisButton.StopDragButton = CreateFrame("Frame","Symbiosis_StopDragButton",SymbiosisButton);
	SymbiosisButton.StopDragButton.tex = SymbiosisButton.StopDragButton:CreateTexture();
	
	local stopdrag = SymbiosisButton.StopDragButton;
	stopdrag:SetWidth(100);
	stopdrag:SetHeight(30);
	stopdrag:SetPoint("BOTTOM",SymbiosisButton,"TOP",0,20);
	stopdrag:SetScript("OnMouseDown",function()
		stopdrag:Hide();
		SymbiosisButton:SetScript("OnDragStart",nil);
		print("Symbiosis: Dragging stoped.");--loc
		Symbiosis.DragButton:SetText("Start Drag");--loc
	end);
	stopdrag:Hide();
	
	local tex = SymbiosisButton.StopDragButton.tex;
	tex:SetTexture(0,.95,0,.7);
	tex:SetAllPoints(stopdrag);
	
	local font = stopdrag:CreateFontString("Symbiosis_StopDragFont","ARTWORK","GameFontNormal");
	font:SetWidth(100);
	font:SetHeight(30);
	font:SetText("Stop Drag");--loc
	font:SetAllPoints(stopdrag);
	font:SetJustifyH("CENTER");
	
	local esc = CreateFrame("BUTTON","Symbiosis_StopDragEscButton",stopdrag,"UIPanelButtonTemplate");
	esc:SetWidth(12);
	esc:SetHeight(12);
	esc:SetText("x");
	esc:SetPoint("TOPRIGHT",-1,-1);
	esc:SetScript("OnClick",function()
		stopdrag:Hide();
	end);
end;

-----------------------------------------
--events
-----------------------------------------

--register event so we notice when combat starts
local CombatFrame = CreateFrame("Frame");
CombatFrame:RegisterEvent("PLAYER_REGEN_ENABLED");
CombatFrame:RegisterEvent("PLAYER_REGEN_DISABLED");
CombatFrame:SetScript("OnEvent",function(self,event,...)
	if event == "PLAYER_REGEN_ENABLED" then
		--do stuff when we leave combat
		if UpdateAfterCombat then
			UpdateAfterCombat = false;
			SetSymbiosisButton();
		end;
		if RemoveAfterCombat then
			RemoveAfterCombat = false;
			SymbiosisButton:SetAttribute("macrotext","");
		end;
	else
		--do stuff when we enter combat
		if pop:IsShown() then
			pop:Hide();
		end;
	end;
end);

--check if Target["unit"] is still the same
local RaidChangedFrame = CreateFrame("Frame");
RaidChangedFrame:RegisterEvent("GROUP_ROSTER_UPDATE");
RaidChangedFrame:SetScript("OnEvent",function()
	--DEBUG:--TODO: check "Unknown" bug
		if symbdebug then
			if GetNumGroupMembers() > 1 then
				for i=1,GetNumGroupMembers() do
					local name,_,subgroup,_,_,class,_,online,isDead = GetRaidRosterInfo(i);
					if not (name or subgroup or class) then
						print("DEBUG: GetRaidRosterInfo("..i..") failed. Returns (subg,name,class,online,isDead) are:",subgroup,name,class,online,isDead);
						print("DEBUG2: Normal functions returns (name,class,online,isDead) are:",UnitName(groupsize..i),UnitClass(groupsize..i),UnitIsConnected(groupsize..i),UnitIsDeadOrGhost(groupsize..i));
					end;
				end
			end
		end;
	--DEBUG END
	
	if (
		( (groupsize == "raid") and (GetNumGroupMembers() ~= membercount) )
	or 	( (groupsize == "party") and (GetNumGroupMembers() ~= membercount) )
	or 	( groupsize == "solo" )
	)
	and ( pop:IsVisible() ) then
		pop:Hide();
	end;

	if (Target["unit"]) and (Target["GUID"]) then
		local GUID = Target["GUID"];
		local unit = Target["unit"];
		local found = false;
		CheckRaidOrParty();
		if not (GUID == UnitGUID(unit)) then
			for i=1,membercount do
				local GUIDTest = UnitGUID(groupsize..i);
				if (GUIDTest == GUID) then					
					Target["unit"] = groupsize..i;
					found = true;
					break;
				end;
			end;
			if found then
				if UnitAffectingCombat("player") then
					UpdateAfterCombat = true;
				else
					SetSymbiosisButton();
				end;
			else
				local name = Target["name"];
				if SymbConfig["TargetConfigDontRemove"] == false then
					Symbiosis.RemoveSymbiosisTarget();
				else
					targetstring:SetText(Target["name"].." (not in raid)");
					targetstring:SetTextColor(1,0,0);
				end;

				if (SymbConfig["TargetConfigNotice"]) and (SymbConfig["TargetConfigDontRemove"] == false) then
					local message = "Symbiosis: Your Symbiosis target left the party";--loc
					if name then
						print(message.." ("..name..").");
					else
						print(message..".");
					end;
				end;
			end;
		else
			if UnitAffectingCombat("player") then
				UpdateAfterCombat = true;
			else
				SetSymbiosisButton();
			end;
		end;
	end;
end);

--register event to check when buffs change
BuffFrame = CreateFrame("Frame");--this is local
BuffFrame:SetScript("OnEvent",function(self,event,unit)
	local name, realm = UnitName(unit);
	if not realm then
		realm = "";
	end;
	if (name == Target["name"]) or ((name.."-"..realm) == Target["name"]) then
		CheckBuffSetTimerAndSetDuration(true);
	end;
end);

--checks if player is in instance. if yes return instance type, one of the following:
    -- "none" when outside an instance
    -- "pvp" when in a battleground
    -- "arena" when in an arena
    -- "party" when in a 5-man instance
    -- "raid" when in a raid instance 
local function GetLocationAndPartyStatus()
	local isininstance, instancetype = IsInInstance();
	if isininstance then
		--return instancetype
		return instancetype;
	else
		--not in instance
		return false;
	end;
end;

local instancetype;
local function InstanceFunction()
	instancetype = GetLocationAndPartyStatus();
	if instancetype then
		if instancetype == "pvp" then
			if SymbConfig["IgnoreBattleground"] == false then
				ShowHideButton(SymbConfig["ShowHideBattleground"]);
			end;
		elseif instancetype == "arena" then
			if SymbConfig["IgnoreArena"] == false then
				ShowHideButton(SymbConfig["ShowHideArena"]);
			end;
		elseif instancetype == "party" then
			if SymbConfig["IgnoreParty"] == false then
				ShowHideButton(SymbConfig["ShowHideParty"]);
			end;
		elseif instancetype == "raid" then
			if SymbConfig["IgnoreRaid"] == false then
				ShowHideButton(SymbConfig["ShowHideRaid"]);
			end;
		else
			if SymbConfig["IgnoreOutside"] == false then
				ShowHideButton(SymbConfig["ShowHideOutside"]);
			end;
		end;
	else
		if SymbConfig["IgnoreOutside"] == false then
			ShowHideButton(SymbConfig["ShowHideOutside"]);
		end;
	end;
end;
local ZoneCheckFrame = CreateFrame("Frame");
ZoneCheckFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
ZoneCheckFrame:SetScript("OnEvent",function()
	--show/hide based on location&config
	InstanceFunction();
end);

--code executed at login
local LoginFrame = CreateFrame("Frame")
LoginFrame:RegisterEvent("PLAYER_LOGIN")
LoginFrame:SetScript("OnEvent",function()
	--setup/create stuff
	Symbiosis.CreateGUI();
	Symbiosis.SetupSavedVars();
	CreateMainButton();
	CreateTargetString();
	CreatePopUpMenu();
	CreateMemberButtons();
	CreateGroupSlider();
	
	--show/hide on login
	InstanceFunction();
	
	--store current specialization
	LastSpec = GetSpecialization();
	
	--stuff when first login
	if SymbConfig["firstlogin"] then
		SymbiosisButton:Show();
		SymbiosisButton:SetScript("OnDragStart",function()
			if SymbiosisButton:IsMovable() then
				SymbiosisButton:StartMoving();
			end;
		end);
		Symbiosis.DragButton:SetText("Stop Drag");--loc
		SymbiosisButton.StopDragButton:Show();
		print("Symbiosis: Welcome ".. UnitName("player") ..". This is your first login with this addon. Drag the Symbiosis button (square thing in the middle of your screen) to where ever you like and hit the 'Stop Drag' button above it.");
	else
		print("Symbiosis: Enabled. Welcome back " .. UnitName("player") .. ".");--loc
	end;
end);

--refresh symbiosis button icon when players changes specs
local SpecChangeFrame = CreateFrame("Frame");
SpecChangeFrame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
SpecChangeFrame:SetScript("OnEvent",function()
	if ( LastSpec ~= GetSpecialization() ) then
		SetSymbiosisButtonIcon();
		LastSpec = GetSpecialization();
	end;
end);