--only startup if player is druid
if Symbiosis.stop then
	return
end

--localized class names
local ClassTable = {};
if UnitSex("player") == 3 then
	FillLocalizedClassList(ClassTable,true);
else
	FillLocalizedClassList(ClassTable);
end;

-----------------------------------------
--Whisper stuff
-----------------------------------------
local StandardMessage = "You have "..GetSpellLink(Symbiosis.SymbiosisSpellID).." now!";
function Symbiosis.GetWhisperMessage(DruidSpec,TargetClass)
	local mymsg = StandardMessage;
	
	if SymbConfig["WhisperAddGrant"] then
		--define locals
		local Choice = false;
		local grantedLink1;
		local grantedLink2;
		local grantedLink3;
		
		--generate spell links
		if Symbiosis.SpellsGranted[TargetClass][1] then
			grantedLink1 = GetSpellLink(Symbiosis.SpellsGranted[TargetClass][1]);
		end;
		if Symbiosis.SpellsGranted[TargetClass][2] then
			grantedLink2 = GetSpellLink(Symbiosis.SpellsGranted[TargetClass][2]);
		end;
		if Symbiosis.SpellsGranted[TargetClass][3] then
			grantedLink3 = GetSpellLink(Symbiosis.SpellsGranted[TargetClass][3]);
		end;
		
		--fill link list
		local grantedLinklist = "";
		if grantedLink1 then
			grantedLinklist = grantedLink1;
		end;
		if grantedLink2 then
			grantedLinklist = grantedLinklist..", "..grantedLink2;
			Choice = true;
		end;
		if grantedLink3 then
			grantedLinklist = grantedLinklist .. ", " .. grantedLink3;
			Choice = true;
		end;
		
		--add link list to msg
		if Choice then
			mymsg = mymsg .. " You will get one of the following spells, based on your spec: " .. grantedLinklist .. ".";
		else
			if grantedLinklist ~= "" then
				mymsg = mymsg .. " You will get the following spell: " .. grantedLinklist .. ".";
			end;
		end;
	end;
	
	if SymbConfig["WhisperAddGet"] then
		local getLink = GetSpellLink(Symbiosis.SpellsGot[DruidSpec][TargetClass]);
		if getLink then
			mymsg = mymsg .. " You granted me " .. getLink .. ".";
		end;
	end;
	
	return mymsg;
end;

-----------------------------------------
--create functions
-----------------------------------------

--create Checkbutton (Checkbox)
local CheckbuttonCount = 0;
local function CreateCheckButton(x_loc,y_loc,displayname,thetooltip,panel)
	CheckbuttonCount = CheckbuttonCount + 1;

	local MyCheckbutton = CreateFrame("CheckButton","Symbiosis_Checkbutton_" .. CheckbuttonCount,panel,"ChatConfigCheckButtonTemplate");
	
	MyCheckbutton:SetPoint("TOPLEFT",x_loc,y_loc);
	_G[MyCheckbutton:GetName() .. "Text"]:SetText(displayname);
	
	local header = "";
	
	if thetooltip ~= "" then
		header = displayname..": ";
	end;
	
	if (header ~= "") and (thetooltip ~= "") then
		MyCheckbutton.tooltip = header..thetooltip;
	end;
	
	return MyCheckbutton;
end

--create Editbox (Edit)
local EditBoxCount = 0;
local function CreateEditBox(Width,Height,x_loc,y_loc,panel)
	EditBoxCount = EditBoxCount + 1;

	local MyEditBox = CreateFrame("EditBox","Symbiosis_EditBox_"..EditBoxCount,panel,"InputBoxTemplate");
	MyEditBox:SetWidth(Width);
	MyEditBox:SetHeight(Height);
	MyEditBox:SetPoint("TOPLEFT",x_loc,y_loc);
	MyEditBox:SetAutoFocus(nil);
	MyEditBox:SetFontObject("GameFontHighlight");
	return MyEditBox;
end

--create Button
local ButtonCount = 0;
local function CreateButton(Width,Height,x_loc,y_loc,MyText,panel)
	ButtonCount = ButtonCount + 1;

	local MyButton = CreateFrame("BUTTON","Symbiosis_Button_"..ButtonCount,panel,"UIPanelButtonTemplate");
	MyButton:SetWidth(Width);
	MyButton:SetHeight(Height);
	MyButton:SetText(MyText);
	MyButton:SetPoint("TOPLEFT",x_loc,y_loc);
	MyButton:Show();
	return MyButton;
end

--create fontstring (label) [frames in popup: without x and y]
local FontStringCount = 0;
function Symbiosis.CreateLabel(parent,text,width,x,y)
	local ofx,ofy;
	if (x) and (y) then
		ofx = x;
		ofy = y;
	else
		ofx =10;
		width = width - ofx;
		ofy = 0;
	end;
	FontStringCount = FontStringCount + 1;
	local MyFontString = parent:CreateFontString("Symbiosis_FontString_"..FontStringCount,"ARTWORK","GameFontNormal");
	MyFontString:SetWidth(width);
	MyFontString:SetHeight(50);
	MyFontString:SetText(text);
	MyFontString:SetPoint("TOPLEFT",parent,"TOPLEFT",ofx,ofy);
	MyFontString:SetJustifyH("LEFT");
	MyFontString:Show();

	return MyFontString;
end;

--create Slider
local SliderCount = 0;
function Symbiosis.CreateSlider(x_loc,y_loc,MyText,minval,maxval,step,startval,panel)
	SliderCount = SliderCount + 1;

	local MySlider = CreateFrame("Slider","Symbiosis_Slider_"..SliderCount,panel,"OptionsSliderTemplate");
	MySlider:ClearAllPoints();
	MySlider:SetPoint("TOPLEFT",x_loc,y_loc);
	MySlider:SetMinMaxValues(minval,maxval);
	MySlider:SetValue(startval);
	MySlider:SetValueStep(step);
	_G[MySlider:GetName() .. "Low"]:SetText(minval);
	_G[MySlider:GetName() .. "High"]:SetText(maxval);
	_G[MySlider:GetName() .. "Text"]:SetText(MyText);
	MySlider:Show();

	return MySlider;
end

--create horizontal line
local HLineCount = 0;
local function CreateHLine(y_loc,panel)
	HLineCount = HLineCount + 1;

	local MyTexture = panel:CreateTexture("Symbiosis_HLine_"..HLineCount);
	MyTexture:SetTexture("Interface\\MailFrame\\UI-MailFrame-InvoiceLine");
	MyTexture:SetPoint("TOPLEFT",10,y_loc);
	MyTexture:SetHeight(60);
	MyTexture:SetWidth(500);
	MyTexture:SetVertexColor(1,1,0.5);
end

--add tooltip to button (permanent)
local function AddTooltip(MyFrameWhoWantsTooltip,text)
	MyFrameWhoWantsTooltip:SetScript("OnEnter",function()
		GameTooltip:SetOwner(MyFrameWhoWantsTooltip,"ANCHOR_CURSOR");
		GameTooltip:SetText(MyFrameWhoWantsTooltip:GetText());

		local LineCount = 0;
		local mytext;
		if type(text) == "string" then
			mytext = {text,"XYZ_NOTHING_XYZ"};
			LineCount = -1;
		else
			mytext = text;
		end

		for _,_ in pairs(mytext) do
			LineCount = LineCount + 1;
		end
		for i=1,LineCount do
			GameTooltip:AddLine(mytext[i],1,1,1);
		end

		GameTooltip:Show();
	end);

	MyFrameWhoWantsTooltip:SetScript("OnLeave",function()
		GameTooltip:Hide();
	end);
end

--add a changing tooltip, use GameTooltip:SetText() and GameTooltip:AddLine()
local function AddTooltipFunc(MyFrameWhoWantsTooltip,TextFunction)
	MyFrameWhoWantsTooltip:SetScript("OnEnter",function()
		GameTooltip:SetOwner(MyFrameWhoWantsTooltip,"ANCHOR_CURSOR");

		TextFunction();

		GameTooltip:Show();
	end);

	MyFrameWhoWantsTooltip:SetScript("OnLeave",function()
		GameTooltip:Hide();
	end);
end

--Adds a spell info line to the GUI/info panel
local ContainerCount = 0;
local ClassFrameCount = 0;
local SpellFrameCount = 0;
local function CreateSpellInfoLine(yoffset,parent,ClassName,SpellId,IsMainLine)
	--------------------------
	--1: Class/spec Icon
	--------------------------
	ClassFrameCount = ClassFrameCount + 1;
	
	local ClassCoords = CLASS_ICON_TCOORDS[ClassName];
	local ClassFrame = CreateFrame("Frame","Symbiosis_SpellInfo_ClassFrame_"..ClassFrameCount,parent);
	ClassFrame:SetHeight(25);
	ClassFrame:SetWidth(25);
	ClassFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",30,-65+yoffset);	
	ClassFrame.icon = ClassFrame:CreateTexture(nil,"CENTER");
	ClassFrame.icon:SetTexture("Interface\\GLUES\\CHARACTERCREATE\\UI-CHARACTERCREATE-CLASSES");
	ClassFrame.icon:SetTexCoord(unpack(ClassCoords));
	ClassFrame.icon:SetAllPoints(ClassFrame);
	
	--------------------------
	--2: Class Name (Text)
	--------------------------
	Symbiosis.CreateLabel(parent,ClassTable[ClassName]..":",100,65,-50+yoffset)

if (not IsMainLine) then
	--------------------------
	--3: Spell Icon
	--------------------------
	SpellFrameCount = SpellFrameCount + 1;
	
	local SpellName,_,SpellIcon = GetSpellInfo(SpellId);
	
	local SpellFrame = CreateFrame("Frame","Symbiosis_SpellInfo_SpellFrame_"..SpellFrameCount,parent);
	SpellFrame:SetHeight(25);
	SpellFrame:SetWidth(25);
	SpellFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",175,-65+yoffset);	
	SpellFrame.icon = SpellFrame:CreateTexture(nil,"CENTER");
	SpellFrame.icon:SetTexture(SpellIcon);
	SpellFrame.icon:SetAllPoints(SpellFrame);
	
	SpellFrame:SetScript("OnEnter",function()
		GameTooltip:SetOwner(SpellFrame,"ANCHOR_CURSOR");
		GameTooltip:SetHyperlink("spell:"..SpellId);
		GameTooltip:Show();
	end);
	
	SpellFrame:SetScript("OnLeave",function()
		GameTooltip:Hide();
	end);
	
	--------------------------
	--4: Spell Name
	--------------------------
	Symbiosis.CreateLabel(parent,SpellName,140,205,-50+yoffset)
end;
end;

local SpecFrameCount = 0;
local function CreateSpecSubLine(yoffset,parent,SpecIcon,SpecName,SpellId)
	--------------------------
	--1: Spec Icon
	--------------------------
	SpecFrameCount = SpecFrameCount + 1;
	local SpecFrame = CreateFrame("Frame","Symbiosis_SpellInfo_SpecFrame_"..SpecFrameCount,parent);
	SpecFrame:SetHeight(25);
	SpecFrame:SetWidth(25);
	SpecFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",60,-65+yoffset);	
	SpecFrame.icon = SpecFrame:CreateTexture(nil,"CENTER");
	SpecFrame.icon:SetTexture("Interface\\Icons\\"..SpecIcon);
	SpecFrame.icon:SetAllPoints(SpecFrame);
	
	--------------------------
	--2: Spec Name (Text)
	--------------------------
	Symbiosis.CreateLabel(parent,SpecName..":",100,100,-50+yoffset)
	
	--------------------------
	--3: Spell Icon
	--------------------------
	SpellFrameCount = SpellFrameCount + 1;
	
	local SpellName,_,SpellIcon = GetSpellInfo(SpellId);
	
	local SpellFrame = CreateFrame("Frame","Symbiosis_SpellInfo_SpellFrame_"..SpellFrameCount,parent);
	SpellFrame:SetHeight(25);
	SpellFrame:SetWidth(25);
	SpellFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",200,-65+yoffset);	
	SpellFrame.icon = SpellFrame:CreateTexture(nil,"CENTER");
	SpellFrame.icon:SetTexture(SpellIcon);
	SpellFrame.icon:SetAllPoints(SpellFrame);
	
	SpellFrame:SetScript("OnEnter",function()
		GameTooltip:SetOwner(SpellFrame,"ANCHOR_CURSOR");
		GameTooltip:SetHyperlink("spell:"..SpellId);
		GameTooltip:Show();
	end);
	
	SpellFrame:SetScript("OnLeave",function()
		GameTooltip:Hide();
	end);
	
	--------------------------
	--4: Spell Name
	--------------------------
	Symbiosis.CreateLabel(parent,SpellName,140,240,-50+yoffset)
end;

-----------------------------------------
--Go and Create GUI
-----------------------------------------

function Symbiosis.CreateGUI()
	-----------------------------------------
	--Locals
	-----------------------------------------
	local StartHeight = 0;
	local currentpanel;
	local PanelStringColor = "|cffFF7D0A";

	-----------------------------------------
	--Main Panel
	-----------------------------------------
	currentpanel = CreateFrame("Frame","SymbiosisMainPanel",UIParent);
	currentpanel.name = "Symbiosis";

	local MainPanelString = Symbiosis.CreateLabel(currentpanel,PanelStringColor.."Symbiosis|r (v"..GetAddOnMetadata("Symbiosis","Version")..")".." options",400,50,StartHeight);--loc
	
	MainPanelString:ClearAllPoints();
	MainPanelString:SetPoint("TOP",currentpanel,"TOP");
	
	Symbiosis.CreateLabel(currentpanel,"Show/Hide when entering...",200,30,StartHeight-80);--loc
	
	local LocDummy = "Status of Symbiosis button in |cffFFFFFF";--loc
	local LocDummy2 = "Status will not change in |cffFFFFFF";--loc
	local LocDummy3 = "Check this box to not change status when entering |cffFFFFFF";--loc
	local function GenerateToolTip(Location,IsShown)
		return LocDummy .. Location .. "|r: " .. IsShown .. "."
	end;
	
	--ARENA SHOW/HIDE
	Symbiosis.ShowHideButtonArena 			=	CreateCheckButton(60,StartHeight-80-45,		"Arena",			"",currentpanel);--loc
	Symbiosis.ShowHideButtonArena:SetChecked(true);
	Symbiosis.ShowHideButtonArena:SetScript("OnClick",function()
		if Symbiosis.ShowHideButtonArena:GetChecked() == 1 then
			SymbConfig["ShowHideArena"] = true;
		else
			SymbConfig["ShowHideArena"] = false;
		end
		GameTooltip:Hide();
	end);
	
	local function ShowHideArenaTTFunc()
		if Symbiosis.ShowHideButtonArena:GetChecked() == 1 then 
			GameTooltip:SetText(GenerateToolTip("Arena","Shown"));--loc
		else
			GameTooltip:SetText(GenerateToolTip("Arena","Hidden"));--loc
		end;
	end;
	AddTooltipFunc(Symbiosis.ShowHideButtonArena,ShowHideArenaTTFunc);
	
	--ARENA IGNORE
	Symbiosis.IgnoreArena 					=	CreateCheckButton(35,StartHeight-80-45,"","",currentpanel);
	Symbiosis.IgnoreArena:SetChecked(false);
	Symbiosis.IgnoreArena:SetScript("OnClick",function()
		if Symbiosis.IgnoreArena:GetChecked() == 1 then
			SymbConfig["IgnoreArena"] = true;
			Symbiosis.ShowHideButtonArena:Disable();
		else
			SymbConfig["IgnoreArena"] = false;
			Symbiosis.ShowHideButtonArena:Enable();
		end;
		GameTooltip:Hide();
	end);
	
	local function IgnoreArenaTTFunc()
		if Symbiosis.IgnoreArena:GetChecked() == 1 then 
			GameTooltip:SetText(LocDummy2 .. "Arena" .. "|r.");--loc
		else
			GameTooltip:SetText(LocDummy3 .. "Arena" .. "|r.");--loc
		end;
	end;
	AddTooltipFunc(Symbiosis.IgnoreArena,IgnoreArenaTTFunc);
	
	--BATTLEGROUND SHOW/HIDE
	Symbiosis.ShowHideButtonBattleground	=	CreateCheckButton(60,StartHeight-80-45-25,"Battleground","",currentpanel);--loc
	Symbiosis.ShowHideButtonBattleground:SetChecked(true);
	Symbiosis.ShowHideButtonBattleground:SetScript("OnClick",function()
		if Symbiosis.ShowHideButtonBattleground:GetChecked() == 1 then
			SymbConfig["ShowHideBattleground"] = true;
		else
			SymbConfig["ShowHideBattleground"] = false;
		end
		GameTooltip:Hide();
	end);
	
	local function ShowHideBattlegroundTTFunc()
		if Symbiosis.ShowHideButtonBattleground:GetChecked() == 1 then 
			GameTooltip:SetText(GenerateToolTip("Battleground","Shown"));--loc
		else
			GameTooltip:SetText(GenerateToolTip("Battleground","Hidden"));--loc
		end;
	end;
	AddTooltipFunc(Symbiosis.ShowHideButtonBattleground,ShowHideBattlegroundTTFunc);
	
	--BATTLEGROUND IGNORE
	Symbiosis.IgnoreBattleground			= 	CreateCheckButton(35,StartHeight-80-45-25,"","",currentpanel);
	Symbiosis.IgnoreBattleground:SetChecked(false);
	Symbiosis.IgnoreBattleground:SetScript("OnClick",function()
		if Symbiosis.IgnoreBattleground:GetChecked() == 1 then
			SymbConfig["IgnoreBattleground"] = true;
			Symbiosis.ShowHideButtonBattleground:Disable();
		else
			SymbConfig["IgnoreBattleground"] = false;
			Symbiosis.ShowHideButtonBattleground:Enable();
		end;
		GameTooltip:Hide();
	end);
	
	local function IgnoreBattlegroundTTFunc()
		if Symbiosis.IgnoreBattleground:GetChecked() == 1 then 
			GameTooltip:SetText(LocDummy2 .. "Battleground" .. "|r.");--loc
		else
			GameTooltip:SetText(LocDummy3 .. "Battleground" .. "|r.");--loc
		end;
	end;
	AddTooltipFunc(Symbiosis.IgnoreBattleground,IgnoreBattlegroundTTFunc);
	
	--RAID SHOW/HIDE
	Symbiosis.ShowHideButtonRaid 			=	CreateCheckButton(60,StartHeight-80-45-50,"Raid instance","",currentpanel);--loc
	Symbiosis.ShowHideButtonRaid:SetChecked(true);
	Symbiosis.ShowHideButtonRaid:SetScript("OnClick",function()
		if Symbiosis.ShowHideButtonRaid:GetChecked() == 1 then
			SymbConfig["ShowHideRaid"] = true;
		else
			SymbConfig["ShowHideRaid"] = false;
		end
		GameTooltip:Hide();
	end);
	
	local function ShowHideRaidTTFunc()
		if Symbiosis.ShowHideButtonRaid:GetChecked() == 1 then 
			GameTooltip:SetText(GenerateToolTip("Raid","Shown"));--loc
		else
			GameTooltip:SetText(GenerateToolTip("Raid","Hidden"));--loc
		end;
	end;
	AddTooltipFunc(Symbiosis.ShowHideButtonRaid,ShowHideRaidTTFunc);
	
	--RAID IGNORE
	Symbiosis.IgnoreRaid 					= 	CreateCheckButton(35,StartHeight-80-45-50,"","",currentpanel);
	Symbiosis.IgnoreRaid:SetChecked(false);
	Symbiosis.IgnoreRaid:SetScript("OnClick",function()
		if Symbiosis.IgnoreRaid:GetChecked() == 1 then
			SymbConfig["IgnoreRaid"] = true;
			Symbiosis.ShowHideButtonRaid:Disable();
		else
			SymbConfig["IgnoreRaid"] = false;
			Symbiosis.ShowHideButtonRaid:Enable();
		end;
		GameTooltip:Hide();
	end);
	
	local function IgnoreRaidTTFunc()
		if Symbiosis.IgnoreRaid:GetChecked() == 1 then 
			GameTooltip:SetText(LocDummy2 .. "Raid" .. "|r.");--loc
		else
			GameTooltip:SetText(LocDummy3 .. "Raid" .. "|r.");--loc
		end;
	end;
	AddTooltipFunc(Symbiosis.IgnoreRaid,IgnoreRaidTTFunc);
	
	--PARTY SHOW/HIDE
	Symbiosis.ShowHideButtonParty 			=	CreateCheckButton(60,StartHeight-80-45-75,"Party instance","",currentpanel);--loc
	Symbiosis.ShowHideButtonParty:SetChecked(true);
	Symbiosis.ShowHideButtonParty:SetScript("OnClick",function()
		if Symbiosis.ShowHideButtonParty:GetChecked() == 1 then
			SymbConfig["ShowHideParty"] = true;
		else
			SymbConfig["ShowHideParty"] = false;
		end
		GameTooltip:Hide();
	end);
	
	local function ShowHidePartyTTFunc()
		if Symbiosis.ShowHideButtonParty:GetChecked() == 1 then 
			GameTooltip:SetText(GenerateToolTip("Party","Shown"));--loc
		else
			GameTooltip:SetText(GenerateToolTip("Party","Hidden"));--loc
		end;
	end;
	AddTooltipFunc(Symbiosis.ShowHideButtonParty,ShowHidePartyTTFunc);
	
	--PARTY IGNORE
	Symbiosis.IgnoreParty					= 	CreateCheckButton(35,StartHeight-80-45-75,"","",currentpanel);
	Symbiosis.IgnoreParty:SetChecked(false);
	Symbiosis.IgnoreParty:SetScript("OnClick",function()
		if Symbiosis.IgnoreParty:GetChecked() == 1 then
			SymbConfig["IgnoreParty"] = true;
			Symbiosis.ShowHideButtonParty:Disable();
		else
			SymbConfig["IgnoreParty"] = false;
			Symbiosis.ShowHideButtonParty:Enable();
		end;
		GameTooltip:Hide();
	end);
	
	local function IgnorePartyTTFunc()
		if Symbiosis.IgnoreParty:GetChecked() == 1 then 
			GameTooltip:SetText(LocDummy2 .. "Party" .. "|r.");--loc
		else
			GameTooltip:SetText(LocDummy3 .. "Party" .. "|r.");--loc
		end;
	end;
	AddTooltipFunc(Symbiosis.IgnoreParty,IgnorePartyTTFunc);
	
	--OUTSIDE SHOW/HIDE
	Symbiosis.ShowHideButtonOutside			=	CreateCheckButton(60,StartHeight-80-45-100,"Outside","",currentpanel);--loc
	Symbiosis.ShowHideButtonOutside:SetChecked(true);
	Symbiosis.ShowHideButtonOutside:SetScript("OnClick",function()
		if Symbiosis.ShowHideButtonOutside:GetChecked() == 1 then
			SymbConfig["ShowHideOutside"] = true;
		else
			SymbConfig["ShowHideOutside"] = false;
		end
		GameTooltip:Hide();
	end);
	
	local function ShowHideOutsideTTFunc()
		if Symbiosis.ShowHideButtonOutside:GetChecked() == 1 then 
			GameTooltip:SetText(GenerateToolTip("Outside","Shown"));--loc
		else
			GameTooltip:SetText(GenerateToolTip("Outside","Hidden"));--loc
		end;
		GameTooltip:AddLine("Everything non instanced is considered 'outside'.",1,1,1);--loc
	end;
	AddTooltipFunc(Symbiosis.ShowHideButtonOutside,ShowHideOutsideTTFunc);
	
	--OUTSIDE IGNORE
	Symbiosis.IgnoreOutside 				= 	CreateCheckButton(35,StartHeight-80-45-100,"","",currentpanel);
	Symbiosis.IgnoreOutside:SetChecked(false);
	Symbiosis.IgnoreOutside:SetScript("OnClick",function()
		if Symbiosis.IgnoreOutside:GetChecked() == 1 then
			SymbConfig["IgnoreOutside"] = true;
			Symbiosis.ShowHideButtonOutside:Disable();
		else
			SymbConfig["IgnoreOutside"] = false;
			Symbiosis.ShowHideButtonOutside:Enable();
		end;
		GameTooltip:Hide();
	end);
	
	local function IgnoreOutsideTTFunc()
		if Symbiosis.IgnoreOutside:GetChecked() == 1 then 
			GameTooltip:SetText(LocDummy2 .. "Outside" .. "|r.");--loc
		else
			GameTooltip:SetText(LocDummy3 .. "Outside" .. "|r.");--loc
		end;
	end;
	AddTooltipFunc(Symbiosis.IgnoreOutside,IgnoreOutsideTTFunc);
	
	--DONT SHOW UNIT TAGGED AS
	Symbiosis.CreateLabel(currentpanel,"Don't show units tagged as...",200,350,StartHeight-80);--loc
	
	Symbiosis.DontShowUnitOffline =			CreateCheckButton(365,StartHeight-80-45,	"Offline","",currentpanel);--loc
	Symbiosis.DontShowUnitOffline:SetChecked(false);
	Symbiosis.DontShowUnitOffline:SetScript("OnClick",function()
		if Symbiosis.DontShowUnitOffline:GetChecked() == 1 then
			SymbConfig["DontShowUnitOffline"] = true;
		else
			SymbConfig["DontShowUnitOffline"] = false;
		end
	end);
	
	Symbiosis.DontShowUnitStolen =			CreateCheckButton(365,StartHeight-80-45-25,	"Stolen","Unit has Symbiosis buff from other druid.",currentpanel);--loc
	Symbiosis.DontShowUnitStolen:SetChecked(false);
	Symbiosis.DontShowUnitStolen:SetScript("OnClick",function()
		if Symbiosis.DontShowUnitStolen:GetChecked() == 1 then
			SymbConfig["DontShowUnitStolen"] = true;
		else
			SymbConfig["DontShowUnitStolen"] = false;
		end
	end);
	
	Symbiosis.DontShowUnitDead =			CreateCheckButton(365,StartHeight-80-45-50,	"Dead","",currentpanel);--loc
	Symbiosis.DontShowUnitDead:SetChecked(false);
	Symbiosis.DontShowUnitDead:SetScript("OnClick",function()
		if Symbiosis.DontShowUnitDead:GetChecked() == 1 then
			SymbConfig["DontShowUnitDead"] = true;
		else
			SymbConfig["DontShowUnitDead"] = false;
		end
	end);
	
	Symbiosis.DontShowUnitInsignificant =	CreateCheckButton(365,StartHeight-80-45-75,	"In insignificant Group","Only scan people in group 1 to X, where X is based on currently selected raid size. Which is X=2 for 10 man raid or X=5 for 25 man raid. To scan all 8 groups disable this option.",currentpanel);--loc
	Symbiosis.DontShowUnitInsignificant:SetChecked(false);
	Symbiosis.DontShowUnitInsignificant:SetScript("OnClick",function()
		if Symbiosis.DontShowUnitInsignificant:GetChecked() == 1 then
			SymbConfig["DontShowUnitInsignificant"] = true;
		else
			SymbConfig["DontShowUnitInsignificant"] = false;
		end;
	end);

	CreateHLine(StartHeight-80-45-100-20,currentpanel);
	
	Symbiosis.CreateLabel(currentpanel,"Target Config",200,30,StartHeight-80-210);
	
	Symbiosis.TargetConfigDontRemove =		CreateCheckButton(45,StartHeight-80-210-45,"Don't remove target if target leaves raid","You wont be able to cast Symbiosis on selected target unless target rejoins raid. This option may be usefull for arena/battleground groups.",currentpanel);
	Symbiosis.TargetConfigDontRemove:SetChecked(false);
	Symbiosis.TargetConfigDontRemove:SetScript("OnClick",function()
		if Symbiosis.TargetConfigDontRemove:GetChecked() == 1 then
			SymbConfig["TargetConfigDontRemove"] = true;
		else
			SymbConfig["TargetConfigDontRemove"] = false;
		end;
	end);
	
	Symbiosis.TargetConfigNotice =			CreateCheckButton(45,StartHeight-80-210-45-25,"Print message when target leaves raid","",currentpanel);
	Symbiosis.TargetConfigNotice:SetChecked(true);
	Symbiosis.TargetConfigNotice:SetScript("OnClick",function()
		if Symbiosis.TargetConfigNotice:GetChecked() == 1 then
			SymbConfig["TargetConfigNotice"] = true;
		else
			SymbConfig["TargetConfigNotice"] = false;
		end;
	end);
	
	Symbiosis.DisableBuffRemove =			CreateCheckButton(45,StartHeight-80-210-45-50,"Disable shift-left-click on Symbiosis button","A shift-left-click on the Symbiosis button removes the current Symbiosis buff.",currentpanel);
	Symbiosis.DisableBuffRemove:SetChecked(false);
	Symbiosis.DisableBuffRemove:SetScript("OnClick",function()
		if Symbiosis.DisableBuffRemove:GetChecked() == 1 then
			SymbConfig["DisableBuffRemove"] = true;
			SymbiosisButton:SetAttribute("shift-type1","");
		else
			SymbConfig["DisableBuffRemove"] = false;
			SymbiosisButton:SetAttribute("shift-type1","cancelaura");
		end;
	end);
	
	CreateHLine(StartHeight-80-210-45-50-30,currentpanel);
	
	Symbiosis.CreateLabel(currentpanel,"Symbiosis Button",200,30,StartHeight-80-210-45-50-30-40);
	
	--------------
	--Sizer Frame
	--------------
	--sizer frame (container)
	local SizerFrame = CreateFrame("Frame","Symbiosis_SizerFrame",UIParent);
	SizerFrame:SetWidth(250);
	SizerFrame:SetHeight(120);
	SizerFrame:SetPoint("CENTER");
	SizerFrame:Hide();

	--enable dragging
	SizerFrame:EnableMouse(true);
	SizerFrame:SetMovable(true);
	SizerFrame:RegisterForDrag("LeftButton");
	SizerFrame:SetScript("OnDragStop",function()
		SizerFrame:StopMovingOrSizing();
	end);
	SizerFrame:SetScript("OnDragStart",function()
		if SizerFrame:IsMovable() then
			SizerFrame:StartMoving();
		end;
	end);

	--artwork
	SizerFrame.bg = SizerFrame:CreateTexture();
	SizerFrame.bg:SetTexture(0,0,0,0.5);
	SizerFrame.bg:SetAllPoints(SizerFrame);
	SizerFrame:SetBackdrop({
		  edgeFile = "Interface/ArenaEnemyFrame/UI-Arena-Border",
		  tile = true,
		  tileSize = 1,
	edgeSize = 3});

	--title line
	local SizerTitle = Symbiosis.CreateLabel(SizerFrame,"Change size of Symbiosis button:",250,10,10);--loc
	
	--slider on sizerframe
	local SizerSlider = Symbiosis.CreateSlider(0,0,"Size: ?",5,300,5,100,SizerFrame);--loc
	SizerSlider:ClearAllPoints();
	SizerSlider:SetPoint("CENTER",0,0);

	SizerSlider:SetScript("OnValueChanged",function()
		SymbiosisButton:SetWidth((SizerSlider:GetValue()/100)*Symbiosis.ButtonSize);
		SymbiosisButton:SetHeight(SymbiosisButton:GetWidth());
		SymbButton["Size"] = SizerSlider:GetValue();
		_G[SizerSlider:GetName()..'Text']:SetText("Size: " .. SymbButton["Size"].."%");--loc
	end);
	Symbiosis.SizerSlider = SizerSlider;

	--"close" button
	local SizerCloseButton = CreateButton(70,25,0,0,"Done",SizerFrame);--loc
	SizerCloseButton:ClearAllPoints();
	SizerCloseButton:SetPoint("BOTTOMRIGHT",-10,10);
	SizerCloseButton:SetScript("OnClick",function()
		SizerFrame:Hide();
		InterfaceOptionsFrame_OpenToCategory(SymbiosisMainPanel);
	end);

	--"change size" button
	local ChangeSizeButton = CreateButton(100,30,45,StartHeight-80-210-45-50-30-40-50,"Change size",currentpanel);
	AddTooltip(ChangeSizeButton,"Change size of the Symbiosis button.");
	
	ChangeSizeButton:SetScript("OnClick",function()
		if not SymbiosisButton:IsVisible() then
			print("Symbiosis: Symbiosis button is now visible.");--loc
			SymbiosisButton:Show();
		end;
		SizerFrame:Show();
		InterfaceOptionsFrame:Hide();
	end);
	
	--Drag "Symbiosis Button" button
	local DragButton = CreateButton(100,30,45+140,StartHeight-80-210-45-50-30-40-50,"Start Drag",currentpanel);--loc
	Symbiosis.DragButton = DragButton;
	
	AddTooltip(DragButton,"Enables/disables dragging of Symbiosis Button.");--loc
	
	DragButton:SetScript("OnClick",function()
		if (DragButton:GetText() == "Start Drag") then--loc
			if not SymbiosisButton:IsVisible() then
				print("Symbiosis: Symbiosis button is now visible.");--loc
				SymbiosisButton:Show();
			end;
			SymbiosisButton:SetScript("OnDragStart",function()
				if SymbiosisButton:IsMovable() then
					SymbiosisButton:StartMoving();
				end;
			end);
			DragButton:SetText("Stop Drag");--loc
			SymbiosisButton.StopDragButton:Show();
			print("Symbiosis: Dragging enabled. Type '/symb dragstop' to disable or hit the 'Stop Drag' button.");--loc
		else
			SymbiosisButton.StopDragButton:Hide();
			print("Symbiosis: Dragging stoped.");--loc
			SymbiosisButton:SetScript("OnDragStart",nil);
			DragButton:SetText("Start Drag");--loc
		end;
	end);
	
	--reset postion button
	local ResetButton = CreateButton(100,30,45+280,StartHeight-80-210-45-50-30-40-50,"Reset Pos.",currentpanel);--loc
	
	AddTooltip(ResetButton,"Reset position of Symbiosis button to screen center.");--loc
	
	ResetButton:SetScript("OnClick",function()
		if not SymbiosisButton:IsVisible() then
			print("Symbiosis: Symbiosis button is now visible.");--loc
			SymbiosisButton:Show();
		end;
		print("Symbiosis: To confirm reseting the position of the Symbiosis button type '/symb reset'.");
	end);
	
	InterfaceOptions_AddCategory(currentpanel);
	
	-----------------------------------------
	--Whisper Panel
	-----------------------------------------	
	currentpanel = CreateFrame("Frame","SymbiosisWhisperPanel",UIParent);
	currentpanel.name = "Whisper";--loc
	currentpanel.parent = SymbiosisMainPanel.name;
	
	local WhisperPanelString = Symbiosis.CreateLabel(currentpanel,PanelStringColor.."Symbiosis|r (v"..GetAddOnMetadata("Symbiosis","Version")..")".." whisper options (NOT WORKING YET!)",400,20,StartHeight);--loc--TODO: remove "not working" tag
	
	WhisperPanelString:ClearAllPoints();
	WhisperPanelString:SetPoint("TOP",currentpanel,"TOP");
	
	Symbiosis.WhisperEnable = CreateCheckButton(20,StartHeight-50,"Enable target whisper","When you cast Symbiosis your target will receive a whisper from you",currentpanel);--loc
	Symbiosis.WhisperEnable:SetChecked(false);
	
	Symbiosis.TestMessageLabel = Symbiosis.CreateLabel(currentpanel,"Test Message:",400,20,StartHeight-80);--loc
	Symbiosis.WhisperPanelTestWhisperString = Symbiosis.CreateLabel(currentpanel,StandardMessage,400,20,StartHeight-120);--loc
	
	Symbiosis.WhisperPanelAddGrant = CreateCheckButton(20,StartHeight-180,"Add spells granted to the target.","Displays all possible spells the target could get",currentpanel);--loc
	Symbiosis.WhisperPanelAddGrant:SetChecked(false);
	Symbiosis.WhisperPanelAddGrant:SetScript("OnClick",function()
		if Symbiosis.WhisperPanelAddGrant:GetChecked() == 1 then
			SymbConfig["WhisperAddGrant"] = true;
		else
			SymbConfig["WhisperAddGrant"] = false;
		end;
		Symbiosis.WhisperPanelTestWhisperString:SetText(Symbiosis.GetWhisperMessage(1,"PALADIN"));
	end);
	
	Symbiosis.WhisperPanelAddGet = CreateCheckButton(20,StartHeight-180-25,"Add spell granted to you.","",currentpanel);--loc
	Symbiosis.WhisperPanelAddGet:SetChecked(false);
	Symbiosis.WhisperPanelAddGet:SetScript("OnClick",function()
		if Symbiosis.WhisperPanelAddGet:GetChecked() == 1 then
			SymbConfig["WhisperAddGet"] = true;
		else
			SymbConfig["WhisperAddGet"] = false;
		end;
		Symbiosis.WhisperPanelTestWhisperString:SetText(Symbiosis.GetWhisperMessage(1,"PALADIN"));
	end);
	
	--OnClick Event for "Enable Whisper" checkbox
	Symbiosis.WhisperEnable:SetScript("OnClick",function()
		if Symbiosis.WhisperEnable:GetChecked() == 1 then
			SymbConfig["WhisperEnable"] = true;
			Symbiosis.WhisperPanelTestWhisperString:Show();
			Symbiosis.TestMessageLabel:Show();
			Symbiosis.WhisperPanelAddGrant:Show();
			Symbiosis.WhisperPanelAddGet:Show();
		else
			SymbConfig["WhisperEnable"] = false;
			Symbiosis.WhisperPanelTestWhisperString:Hide();
			Symbiosis.TestMessageLabel:Hide();
			Symbiosis.WhisperPanelAddGrant:Hide();
			Symbiosis.WhisperPanelAddGet:Hide();
		end;
	end);

	InterfaceOptions_AddCategory(currentpanel);
	
	-----------------------------------------
	--Spell Info Panel
	-----------------------------------------
	local SpellInfoButtonsStartYPos = -20;
	
	currentpanel = CreateFrame("Frame","SymbiosisSpellInfoPanel",UIParent);
	currentpanel.name = "Info";--loc
	currentpanel.parent = SymbiosisMainPanel.name;
	
	local SpellInfoString = Symbiosis.CreateLabel(currentpanel,PanelStringColor.."Symbiosis|r (v"..GetAddOnMetadata("Symbiosis","Version")..")".." spell infos",400,20,StartHeight);--loc
	
	SpellInfoString:ClearAllPoints();
	SpellInfoString:SetPoint("TOP",currentpanel,"TOP");
	
	-----------
	--setup stuff for what druid GETS

	--moonkin
	local CurrentYOffSet = SpellInfoButtonsStartYPos;
	local MoonkinContainer = CreateFrame("Frame","Symbiosis_SpellInfo_ContainerFrame_Moonkin",currentpanel);
	MoonkinContainer:SetAllPoints(currentpanel);
	MoonkinContainer:Hide();
	for TargetClass,SpellId in pairs(Symbiosis.SpellsGot[1]) do
		CreateSpellInfoLine(CurrentYOffSet,MoonkinContainer,TargetClass,SpellId);
		CurrentYOffSet = CurrentYOffSet - 30;
	end;
	
	--feral
	CurrentYOffSet = SpellInfoButtonsStartYPos;
	local FeralContainer = CreateFrame("Frame","Symbiosis_SpellInfo_ContainerFrame_Feral",currentpanel);
	FeralContainer:SetAllPoints(currentpanel);
	FeralContainer:Hide();
	for TargetClass,SpellId in pairs(Symbiosis.SpellsGot[2]) do
		CreateSpellInfoLine(CurrentYOffSet,FeralContainer,TargetClass,SpellId);
		CurrentYOffSet = CurrentYOffSet - 30;
	end;
	
	--guardian
	CurrentYOffSet = SpellInfoButtonsStartYPos;
	local GuardianContainer = CreateFrame("Frame","Symbiosis_SpellInfo_ContainerFrame_Guardian",currentpanel);
	GuardianContainer:SetAllPoints(currentpanel);
	GuardianContainer:Hide();
	for TargetClass,SpellId in pairs(Symbiosis.SpellsGot[3]) do
		CreateSpellInfoLine(CurrentYOffSet,GuardianContainer,TargetClass,SpellId);
		CurrentYOffSet = CurrentYOffSet - 30;
	end;
	
	--resto
	CurrentYOffSet = SpellInfoButtonsStartYPos;
	local RestoContainer = CreateFrame("Frame","Symbiosis_SpellInfo_ContainerFrame_Resto",currentpanel);
	RestoContainer:SetAllPoints(currentpanel);
	RestoContainer:Hide();
	for TargetClass,SpellId in pairs(Symbiosis.SpellsGot[4]) do
		CreateSpellInfoLine(CurrentYOffSet,RestoContainer,TargetClass,SpellId);
		CurrentYOffSet = CurrentYOffSet - 30;
	end;
	
	--Show requested spell info buttons, hide others
	local function ShowHideSpellButtons(DruidSpec)	
		if (DruidSpec == "Moonkin") then
			MoonkinContainer:Show();
		else
			MoonkinContainer:Hide();
		end;
		
		if (DruidSpec == "Feral") then
			FeralContainer:Show();
		else
			FeralContainer:Hide();
		end;
		
		if (DruidSpec == "Guardian") then
			GuardianContainer:Show();
		else
			GuardianContainer:Hide();
		end;
		
		if (DruidSpec == "Resto") then
			RestoContainer:Show();
		else
			RestoContainer:Hide();
		end;
	end;

	--create 4 buttons to show different spell buttons
	local ShowMoonkinSpells = CreateButton(80,30,30,-40,"Moonkin",currentpanel);--loc
	local ShowFeralSpells = CreateButton(80,30,30+90,-40,"Feral",currentpanel);--loc
	local ShowGuardianSpells = CreateButton(80,30,30+180,-40,"Guardian",currentpanel);--loc
	local ShowRestoSpells = CreateButton(80,30,30+270,-40,"Resto",currentpanel);--loc

	--setup onclick events of the 4 buttons
	ShowMoonkinSpells:SetScript("OnClick",function()
		ShowHideSpellButtons("Moonkin");
	end);

	ShowFeralSpells:SetScript("OnClick",function()
		ShowHideSpellButtons("Feral");
	end);

	ShowGuardianSpells:SetScript("OnClick",function()
		ShowHideSpellButtons("Guardian");
	end);

	ShowRestoSpells:SetScript("OnClick",function()
		ShowHideSpellButtons("Resto");
	end);

	InterfaceOptions_AddCategory(currentpanel);
	
	-----------------------------------------
	--Spell Info Panel2
	-----------------------------------------
	
	currentpanel = CreateFrame("Frame","SymbiosisSpellInfo2Panel",UIParent);
	currentpanel.name = "Info2";--loc
	currentpanel.parent = SymbiosisMainPanel.name;
	
	local SpellInfoString = Symbiosis.CreateLabel(currentpanel,PanelStringColor.."Symbiosis|r (v"..GetAddOnMetadata("Symbiosis","Version")..")".." spell infos2",400,20,StartHeight);--loc
	
	SpellInfoString:ClearAllPoints();
	SpellInfoString:SetPoint("TOP",currentpanel,"TOP");
	
	-----------
	--setup stuff for what druid GRANTS
	local TarClass

	--Grant1: DEATHKNIGHT, HUNTER, MAGE
	CurrentYOffSet = SpellInfoButtonsStartYPos;
	local GrantContainer1 = CreateFrame("Frame","Symbiosis_SpellInfo_ContainerFrame_Grant1",currentpanel);
	GrantContainer1:SetAllPoints(currentpanel);
	GrantContainer1:Hide();
	
	for i=1,3 do
		TarClass = Symbiosis.ClassNames[i];
		--Main Line
		CreateSpellInfoLine(CurrentYOffSet,GrantContainer1,TarClass,nil,true);
		CurrentYOffSet = CurrentYOffSet - 30;
		--Sub Line
		for TarSpec,SpecIcon in pairs(Symbiosis.SpecIcons[TarClass]) do
			CreateSpecSubLine(CurrentYOffSet,GrantContainer1,SpecIcon,Symbiosis.SpecNames[TarClass][TarSpec],Symbiosis.SpellsGranted[TarClass][TarSpec]);
			CurrentYOffSet = CurrentYOffSet - 30;
		end;
		CurrentYOffSet = CurrentYOffSet - 20;
	end;
	
	--Grant2: MONK, PALADIN, PRIEST
	CurrentYOffSet = SpellInfoButtonsStartYPos;
	local GrantContainer2 = CreateFrame("Frame","Symbiosis_SpellInfo_ContainerFrame_Grant2",currentpanel);
	GrantContainer2:SetAllPoints(currentpanel);
	GrantContainer2:Hide();
	
	for i=4,6 do
		TarClass = Symbiosis.ClassNames[i];
		--Main Line
		CreateSpellInfoLine(CurrentYOffSet,GrantContainer2,TarClass,nil,true);
		CurrentYOffSet = CurrentYOffSet - 30;
		--Sub Line
		for TarSpec,SpecIcon in pairs(Symbiosis.SpecIcons[TarClass]) do
			CreateSpecSubLine(CurrentYOffSet,GrantContainer2,SpecIcon,Symbiosis.SpecNames[TarClass][TarSpec],Symbiosis.SpellsGranted[TarClass][TarSpec]);
			CurrentYOffSet = CurrentYOffSet - 30;
		end;
		CurrentYOffSet = CurrentYOffSet - 20;
	end;
	
	--Grant3: ROGUE, SHAMAN, WARLOCK
	CurrentYOffSet = SpellInfoButtonsStartYPos;
	local GrantContainer3 = CreateFrame("Frame","Symbiosis_SpellInfo_ContainerFrame_Grant3",currentpanel);
	GrantContainer3:SetAllPoints(currentpanel);
	GrantContainer3:Hide();
	
	for i=7,9 do
		TarClass = Symbiosis.ClassNames[i];
		--Main Line
		CreateSpellInfoLine(CurrentYOffSet,GrantContainer3,TarClass,nil,true);
		CurrentYOffSet = CurrentYOffSet - 30;
		--Sub Line
		for TarSpec,SpecIcon in pairs(Symbiosis.SpecIcons[TarClass]) do
			CreateSpecSubLine(CurrentYOffSet,GrantContainer3,SpecIcon,Symbiosis.SpecNames[TarClass][TarSpec],Symbiosis.SpellsGranted[TarClass][TarSpec]);
			CurrentYOffSet = CurrentYOffSet - 30;
		end;
		CurrentYOffSet = CurrentYOffSet - 20;
	end;
	
	--Grant4: WARRIOR
	CurrentYOffSet = SpellInfoButtonsStartYPos;
	local GrantContainer4 = CreateFrame("Frame","Symbiosis_SpellInfo_ContainerFrame_Grant4",currentpanel);
	GrantContainer4:SetAllPoints(currentpanel);
	GrantContainer4:Hide();
	
	for i=10,10 do
		TarClass = Symbiosis.ClassNames[i];
		--Main Line
		CreateSpellInfoLine(CurrentYOffSet,GrantContainer4,TarClass,nil,true);
		CurrentYOffSet = CurrentYOffSet - 30;
		--Sub Line
		for TarSpec,SpecIcon in pairs(Symbiosis.SpecIcons[TarClass]) do
			CreateSpecSubLine(CurrentYOffSet,GrantContainer4,SpecIcon,Symbiosis.SpecNames[TarClass][TarSpec],Symbiosis.SpellsGranted[TarClass][TarSpec]);
			CurrentYOffSet = CurrentYOffSet - 30;
		end;
		CurrentYOffSet = CurrentYOffSet - 20;
	end;
	
	--create 4 buttons to show different spell buttons
	local ShowGrant1 = CreateButton(80,30,30,-40,"1",currentpanel);--loc
	local ShowGrant2 = CreateButton(80,30,30+90,-40,"2",currentpanel);--loc
	local ShowGrant3 = CreateButton(80,30,30+180,-40,"3",currentpanel);--loc
	local ShowGrant4 = CreateButton(80,30,30+270,-40,"4",currentpanel);--loc

	--show requested classes
	local function ShowGrantContainer(i)
		if i == 1 then
			GrantContainer1:Show();
		else
			GrantContainer1:Hide();
		end;
		
		if i == 2 then
			GrantContainer2:Show();
		else
			GrantContainer2:Hide();
		end;
		
		if i == 3 then
			GrantContainer3:Show();
		else
			GrantContainer3:Hide();
		end;
		
		if i == 4 then
			GrantContainer4:Show();
		else
			GrantContainer4:Hide();
		end;
	end;
	
	--setup onclick events of the 3 buttons
	ShowGrant1:SetScript("OnClick",function()
		ShowGrantContainer(1);
	end);

	ShowGrant2:SetScript("OnClick",function()
		ShowGrantContainer(2);
	end);

	ShowGrant3:SetScript("OnClick",function()
		ShowGrantContainer(3);
	end);
	
	ShowGrant4:SetScript("OnClick",function()
		ShowGrantContainer(4);
	end);
	
	InterfaceOptions_AddCategory(currentpanel);
end;